from leapp.actors import Actor
from leapp.dialogs import Dialog
from leapp.dialogs.components import BooleanComponent
from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag
from leapp.models import InstalledRPM
from leapp import reporting
from leapp.reporting import Report, create_report
import os
import subprocess


class CheckOSMH(Actor):
    """
    Check if OS Management Hub (OSMH) is configured and active. If yes, inhibit the upgrade process.
    Do not inhibit, if it is auto-detected that instance is the OCI Managed Instance.
    """

    name = 'check_osmh'
    consumes = (InstalledRPM,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        if os.getenv('LEAPP_OSMH') == '1':
            result = self.osmh_check()
            if result == 0:
                self.produce_inhibitor_no_agent()
            return
        if has_package(InstalledRPM, 'oracle-cloud-agent'):
            api.current_logger().warning('oracle-cloud-agent package has been detected')
        if has_package(InstalledRPM, 'oracle.mgmt_agent'):
            api.current_logger().warning('oracle.mgmt_agent package has been detected')
        result = self.osmh_check()
        if result == 1:
            self.produce_inhibitor()

    def osmh_check(self):
        try:
            ps = subprocess.Popen("ps -eaf | grep oci-osmh | grep -v grep", shell=True, stdout=subprocess.PIPE)
            output = ps.stdout.read()
            ps.stdout.close()
            ps.wait()

            if (output.find(b'oci-osmh') != -1):
                api.current_logger().warning('Active OSMH plugin has been detected')
                result = 1
            else:
                result = 0
        except:
            api.current_logger().warning('Unable to determine if OSMH plugin is active')
            result = 1

        return result

    def produce_inhibitor(self):
        remediation = ('The OSMH agent is included in OCI Oracle Linux platform images '
                       'and installed by default. non-OCI Oracle Linux systems may have OSMH agent installed as well. '
                       'If the instance is not actively managed by OSMH, you should disable OSMH agent '
                       'from the OCI console and the upgrade can proceed. Please refer to '
                       'https://docs.oracle.com/en/operating-systems/oracle-linux/9/leapp/ '
                       'for information about upgrading instances managed by OSMH.')
        create_report([
            reporting.Title('The OS Management Hub (OSMH) agent is running on this instance and instance is not identified as managed'),
            reporting.Summary(
                'The OSMH service is active on this instance.\n\n'
                'Upgrade cannot proceed with the OSMH agent active, unless you ensure '
                'yum reports system as receiving updates from OSMH. '
                'Please refer to https://docs.oracle.com/en/operating-systems/oracle-linux/9/leapp/ '
                'for information about upgrading instances managed by OSMH. '
                'Please refer to https://docs.oracle.com/en-us/iaas/osmh/doc/getstarted.htm '
                'for more information about OSMH.'
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([
                    reporting.Groups.SERVICES,
                    reporting.Groups.TOOLS
            ]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Remediation(hint=remediation)
        ])

    def produce_inhibitor_no_agent(self):
        remediation = ('The OSMH agent is not currently running, but system is detected as managed by OSMH. \n'
                       'If you want upgraded instance to be available to be managed by OS Management Hub post '
                       'in-place upgrade completion you need to keep instance as fully registered, actively managed by OS Management Hub and '
                       'with OSMH agent running.\n'
                       'If the instance is not actively managed by OSMH, you should de-register instance as managed by OS Management Hub '
                       'from the OCI console and the upgrade can proceed. \n'
                       'Note: formerly registered instance, will not be available for registration to be managed by OS Management Hub post '
                       'in-place upgrade completion. \n'
                       'Please refer to https://docs.oracle.com/en/operating-systems/oracle-linux/9/leapp/ '
                       'for information about upgrading instances managed by OSMH.')
        create_report([
            reporting.Title('The OS Management Hub (OSMH) agent is not running on this instance and instance is identified as managed'),
            reporting.Summary(
                'The OSMH service is not active on this instance.\n\n'
                'Upgrade cannot proceed with the OSMH agent inactive, unless you ensure '
                'you have de-registered instance as managed by OS Management Hub. \n'
                'Please refer to https://docs.oracle.com/en/operating-systems/oracle-linux/9/leapp/ '
                'for information about upgrading instances managed by OSMH. \n'
                'Please refer to https://docs.oracle.com/en-us/iaas/osmh/doc/getstarted.htm '
                'for more information about OSMH.\n'
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([
                    reporting.Groups.SERVICES,
                    reporting.Groups.TOOLS
            ]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Remediation(hint=remediation)
        ])
