from leapp.libraries.common.config import get_env
from leapp import reporting
from leapp.actors import Actor
from leapp.dialogs import Dialog
from leapp.dialogs.components import BooleanComponent
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag
import os
import subprocess

class OSMHCheck(Actor):
    """
    Check with user that they understand limitations of managed instance upgrade.

    If admin does not confirm that they understand pre-requisites and actions needed
    to perform post-upgrade upgrade will be stalled.
    """
    name = 'osmh_check'
    consumes = ()
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)
    dialogs = (
        Dialog(
            scope='os_management_hub_agent_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that no package updates will be installed '
                          'during system upgrade with OS Management Hub ? ',
                    description='This system is managed by the OS Management Hub service '
                                'and the agent is running. '
                                'Confirm that no updates will be installed during the system upgrade.',
                    reason='Installing any package during the managed instance upgrade '
                           'may leave the system in non-upgradeable state or '
                           'break the upgrade itself.'
                ),
            )
        ),
        Dialog(
            scope='osmh_post_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that you will need to attach Software Sources '
                          ' in OS Management Hub after the upgrade ? '
                          ' You can use the OCI console, CLI or SDK to attach software sources. ',
                    description=' Attaching software sources to the managed instance '
                                ' does not happen automatically.'
                                ' The user needs to confirm they understand'
                                ' that some actions are needed post-upgrade',
                    reason='After upgrade, the managed instance will not have any attached software sources. '
                           'If the post-upgrade steps are not performed, the upgraded instance will not have access to updates and patches.'
                ),
            )
        ),
        Dialog(
            scope='os_management_hub_agent_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that no package updates will be installed '
                          'during system upgrade with OS Management Hub ? ',
                    description='This system is managed by the OS Management Hub service '
                                'and the agent is running. '
                                'Confirm that no updates will be installed during the system upgrade.',
                    reason='Installing any package during the managed instance upgrade '
                           'may leave the system in non-upgradeable state or '
                           'break the upgrade itself.'
                ),
            )
        ),
        Dialog(
            scope='os_management_hub_agent_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that no package updates will be installed '
                          'during system upgrade with OS Management Hub ? ',
                    description='This system is managed by the OS Management Hub service '
                                'but running agent was not detected. '
                                'Confirm that no updates will be installed during the system upgrade.',
                    reason='Installing any package during the managed instance upgrade '
                           'may leave the system in non-upgradeable state or '
                           'break the upgrade itself.'
                ),
            )
        ),
        Dialog(
            scope='osmh_sanity_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that if your instance is a previously managed by OSMH instance, which was de-registered, '
                          ' you will not be able to register it to be managed by OSMH after completing in-place upgrade ? ',
                    description='If system was managed by OSMH, but was de-registered, it will not be possible to register '
                                'it again after in-place upgrade is completed. '
                                'Confirm that either instance is currently registered to be managed by OSMH before upgrade, '
                                'or you will not be registering instance to be managed by OSMH post-upgrade.',
                    reason='Attempting to ugrade instance, that was formerly registered to be managed by OSMH, but currently is not, '
                           'will lead to inability to register upgraded instance to be managed by OSMH '
                ),
            )
        ),
    )

    def osmh_check(self):
        try:
            ps = subprocess.Popen("ps -eaf | grep oci-osmh | grep -v grep", shell=True, stdout=subprocess.PIPE)
            output = ps.stdout.read()
            ps.stdout.close()
            ps.wait()

            if (output.find(b'oci-osmh') != -1):
                result = 1
            else:
                result = 0
        except:
            result = 1
        return result

    def process(self):
        osmh_plugin_status = self.osmh_check()
        service_name = get_env('LEAPP_OSMH_SERVICE')
        osmh_enabled = get_env('LEAPP_OSMH')

        if osmh_enabled and osmh_plugin_status == 1:
            confirmations = ["os_management_hub_agent_check", "osmh_post_check"]
            for confirmation in confirmations:
                result = self.confirm(confirmation)
                if result and confirmation == "os_management_hub_agent_check":
                    self.produce_report_cloud_agent()
                elif result and confirmation == "osmh_post_check":
                    self.produce_report_osmh()
                elif result is False:
                    # user specifically chose to disagree with auto disablement
                    self.produce_inhibitor(confirmation)
        if (osmh_enabled and osmh_plugin_status == 0) or (osmh_enabled and not os.path.isfile("/etc/yum.repos.d/osmh.repo")):
            result = self.confirm("osmh_sanity_check")
            if result:
                self.produce_report_osmh_sanity_check()
        else:
            return

    def confirm(self, confirmations):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        if service_name == "oracle-cloud-agent.service":
          dialognum=0
        elif service_name =="mgmt_agent.service":
          dialognum=2
        else:
          dialognum=3
        questions = {
            'os_management_hub_agent_check': self.dialogs[dialognum],
            'osmh_post_check': self.dialogs[1],
            'osmh_sanity_check': self.dialogs[4],
        }

        return self.get_answers(questions[confirmations]).get('confirm')

    def produce_report_osmh(self):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        create_report([
            reporting.Title('OSMH managed instance upgrade requires user to accept certain software sources requirements.'),
            reporting.Summary(
                'User needs to confirm that they will need to add new software sources '
                'for OSMH managed instance as needed.'

            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Remediation(hint='Do not forget to ensure accepted requirements are met'),
            reporting.RelatedResource('service', service_name)
        ])

    def produce_report_cloud_agent(self):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        create_report([
            reporting.Title('OSMH managed instance upgrade requires user to accept specific package update requirements for OS Management Hub'),
            reporting.Summary(
                ' When managed instance is being upgraded, user needs to ensure that'
                ' no packages will be installed/removed/updated via OS Management Hub during the upgrade'
                ' itself. '

            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Remediation(hint='Do not forget to ensure accepted requirements are met'),
            reporting.RelatedResource('service', service_name)
        ])

    def produce_report_osmh_sanity_check(self):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        create_report([
            reporting.Title('Upgrade of instance, identified as OSMH managed, requires user to accept specific upgrade condition requirements for OS Management Hub'),
            reporting.Summary(
                ' When instance, idenfitied as  OSMH managed, is being upgraded, user needs to ensure that'
                ' either instance is registered as managed by OSMH in OS Management Hub,'
                ' or that after in-place upgrade registration will not be possible. '

            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Remediation(hint='Do not forget to ensure accepted requirements are met'),
            reporting.RelatedResource('service', service_name)
        ])

    def produce_inhibitor(self, reason):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        if reason == "os_management_hub_agent_check":
            reported = "Not installing packages using OS Management Hub during the upgrade process"
        elif reason == "osmh_post_check":
            reported = "User will need to manually add/adjust profile after upgrade"
        elif reason == "osmh_sanity_check":
            reported = "Upgrade conditions of instnace, reported as managed by OSMH, are not accepted"
        create_report([
            reporting.Title(
                'Upgrade process was interrupted because user did not confirm  '
                'he complies with necessary pre or post steps, specifically - '
                '{0}.'.format(reported)),
            reporting.Summary(
                ' Upgrading managed instance requires user to ensure'
                'that packages are not installed using OS Management Hub during the upgrade process.'
                'Also user needs to confirm that they understand that OSMH software sources are to be added manually '
                'after upgrade is finished. '
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Remediation(
                hint='Confirm that you understand requirements to perform manage instance upgrade.'.format(module)),
            reporting.RelatedResource('service', '{0}')
        ])
